unit uDemo;

interface

Uses
	SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
	Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Spin, Menus;

{$IFNDEF WIN32}
	type ShortString = string;
{$ENDIF}

type
	TFrmDemo = class(TForm)
    ColorDialog1: TColorDialog;
		Panelinfo: TPanel;
    Notebook1: TNotebook;
    Image1: TImage;
		Globe: TGlobe;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Options1: TMenuItem;
    ShowGrid1: TMenuItem;
    Gridinfront1: TMenuItem;
    N1: TMenuItem;
    MercatorProjection1: TMenuItem;
    Overlay11: TMenuItem;
		Overlay21: TMenuItem;
		N2: TMenuItem;
		Exit1: TMenuItem;
		Help1: TMenuItem;
		Overlay31: TMenuItem;
		OpenDialog1: TOpenDialog;
		SaveDialog1: TSaveDialog;
    BaseGlobeFile1: TMenuItem;
    Save1: TMenuItem;
		N3: TMenuItem;
    ScrollBox1: TScrollBox;
		GroupBox2: TGroupBox;
    btnFill: TBitBtn;
		btnPen: TBitBtn;
    EditTitle: TEdit;
    btnDelete: TBitBtn;
    GroupBox4: TGroupBox;
    cbxLayer0: TCheckBox;
    cbxLayer1: TCheckBox;
    cbxLayer2: TCheckBox;
		cbxLayer3: TCheckBox;
		cbxLayer4: TCheckBox;
		GroupBox3: TGroupBox;
		btnUp: TSpeedButton;
		btnLeft: TSpeedButton;
		btnExtents: TSpeedButton;
		btnRight: TSpeedButton;
		btnDown: TSpeedButton;
		Label2: TLabel;
		Label3: TLabel;
		SpinLongitude: TSpinEdit;
		SpinLatitude: TSpinEdit;
		btnBitmap: TBitBtn;
		spnZoom: TSpinButton;
		Label1: TLabel;
		lstLocations: TListBox;
		GroupBox5: TGroupBox;
		cbxPolygon: TCheckBox;
		cbxPolyline: TCheckBox;
		cbxPoint: TCheckBox;
		N4: TMenuItem;
		ShowHints1: TMenuItem;
		MemoHelp: TMemo;
		Panel1: TPanel;
		Button1: TButton;
		Image2: TImage;
		procedure GlobeMouseDown(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure GlobeMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
		procedure cbxLayer0Click(Sender: TObject);
		procedure EditTitleChange(Sender: TObject);
		procedure btnDeleteClick(Sender: TObject);
		procedure btnUpClick(Sender: TObject);
		procedure btnDownClick(Sender: TObject);
		procedure btnRightClick(Sender: TObject);
		procedure btnLeftClick(Sender: TObject);
		procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
		procedure SpinLatLongChange(Sender: TObject);
		procedure btnFillClick(Sender: TObject);
		procedure btnPenClick(Sender: TObject);
		procedure btnExtentsClick(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure GlobeRender(Sender: TObject);
		procedure Button1Click(Sender: TObject);
		procedure GlobeZoomed(Sender: TObject);
		procedure Image1Click(Sender: TObject);
		procedure MercatorProjection1Click(Sender: TObject);
		procedure ShowGrid1Click(Sender: TObject);
		procedure Gridinfront1Click(Sender: TObject);
		procedure Exit1Click(Sender: TObject);
		procedure LoadOverlayClick(Sender: TObject);
		procedure Save1Click(Sender: TObject);
		procedure GlobeMouseUp(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure btnBitmapClick(Sender: TObject);
		procedure spnZoomDownClick(Sender: TObject);
		procedure spnZoomUpClick(Sender: TObject);
		procedure lstLocationsClick(Sender: TObject);
		procedure UpdateLocationList(Sender: TObject);
		procedure GlobeSelected(Sender: TObject);
		procedure ShowHints1Click(Sender: TObject);
		procedure Help1Click(Sender: TObject);
	private
		{ Private declarations }
	public
		{ Public declarations }
		procedure IdleHandler(Sender: TObject; var Done: Boolean);
		procedure UpdateGlobe;
		function ShowMousePosition : TPoint;
	end;

	{---------------------------- TGlobeTriangle --------------------------------}
	TGlobeTriangle = class(TGlobeObject)
	public
		FPen : TGlobePen;
		FBrush : TGlobeBrush;
		FFont : TGlobeFont;
		constructor Create( Overlay : TGlobeOverlay; const sTitle : string;
			iLayer : integer; iLong, iLat : integer );
		procedure Render; override;
		procedure CalcProjectionData; override;
		procedure LoadFromStream( AStream : TStream ); override;
		procedure SaveToStream( AStream : TStream ); override;
	end;

{-------------------------------------------------------------------------}
var
	FrmDemo : TFrmDemo;
	iLong : integer;
	iMouseX, iMouseY : integer;
	bMouseDown : boolean;
	ptLoc : TPoint;
	MyObject : TGlobeObject;

{-------------------------------------------------------------------------}

implementation

{$R *.DFM}

{-------------------------------------------------------------------------
	TGlobeTriangle.Create
-------------------------------------------------------------------------}
constructor TGlobeTriangle.Create( Overlay : TGlobeOverlay; const sTitle : string;
	iLayer : integer; iLong, iLat : integer );
begin
	inherited Create( Overlay, sTitle, iLayer, iLong, iLat );

	with Overlay.Globe do
	begin
		FPen := TGlobePen.Define( Overlay, Pen.Color, Pen.Style );
		FBrush := TGlobeBrush.Define( Overlay, Brush.Color, Brush.Style );
		FFont := TGlobeFont.Define( Overlay, 'Arial', Font.Color, Font.Size, 0, Font.Style );
	end;
end;

{-------------------------------------------------------------------------
	TGlobeTriangle.Calcprojectiondata
-------------------------------------------------------------------------}
procedure TGlobeTriangle.Calcprojectiondata;
begin
	with Origin do
		BoundsRect := Rect( X - 30, Y - 30, X + 30, Y + 30 );
end;

{-------------------------------------------------------------------------
	TGlobeTriangle.Render
-------------------------------------------------------------------------}
procedure TGlobeTriangle.Render;
var
	iTmp : integer;
	pt : TPoint;
begin
	with Overlay.Globe do
	begin
		RenderAttributes( State, [ FPen, FBrush, FFont ] );

		LLToDeviceXY( Origin.X, Origin.Y, pt );
		with pt do
		begin
			{ Scale the size to the globe }
			iTmp := ScaleMinutesToDevice( 30 ) + 1;

			GlobeCanvas.Polygon( [
				Point( X , Y + iTmp ), Point( X - iTmp, Y - iTmp ), Point( X + iTmp, Y - iTmp ) ] );

			SetBkMode( GlobeCanvas.Handle, TRANSPARENT );
			GlobeCanvas.TextOut( X - iTmp, Y + iTmp, Title );
			SetBkMode( GlobeCanvas.Handle, OPAQUE );
		end;
	end;
end;

{-------------------------------------------------------------------------
	TGlobeTriangle.LoadFromStream
-------------------------------------------------------------------------}
procedure TGlobeTriangle.LoadFromStream( AStream : TStream );
	{-------------------------------}
	function ReadSmallInt : SmallInt;
	begin
		AStream.Read( Result, SizeOf( SmallInt ));
	end;
begin
	inherited LoadFromStream( AStream );

	FPen := TGlobePen( Overlay.Attributes[ReadSmallInt] );
	FBrush := TGlobeBrush( Overlay.Attributes[ReadSmallInt] );
	FFont := TGlobeFont( Overlay.Attributes[ReadSmallInt] );
end;

{-------------------------------------------------------------------------
	TGlobeTriangle.SaveToStream
-------------------------------------------------------------------------}
procedure TGlobeTriangle.SaveToStream( AStream : TStream );
	{-----------------------------------------}
	procedure WriteSmallInt( iVal : SmallInt );
	begin
		AStream.Write( iVal, SizeOf( SmallInt ));
	end;
begin
	inherited SaveToStream( AStream );

	{ only save the references to the attributes }
	WriteSmallInt( FPen.Index );
	WriteSmallInt( FBrush.Index );
	WriteSmallInt( FFont.Index );
end;

{-------------------------------------------------------------------------
	TFrmDemo.ShowMousePosition
-------------------------------------------------------------------------}
function TFrmDemo.ShowMousePosition : TPoint;
var
	sTmp : string;
	MouseObj : TGlobeObject;
begin
	with Globe do
	begin
		MouseObj := ObjectAtXY( iMouseX, iMouseY );
		if MouseObj <> nil then
			sTmp := MouseObj.Title
		else
			sTmp := '';

		DeviceXYToLL( iMouseX, iMouseY, Result );
		Caption := Format( '%s %s', [LLAsString( Result.X, Result.Y ), sTmp] );
	end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.FormCreate
-------------------------------------------------------------------------}
procedure TFrmDemo.FormCreate(Sender: TObject);
begin
	iLong := 0;

	Globe.RegisterGlobeClass( TGlobeTriangle );
	Application.OnIdle := IdleHandler;
	Show;

	Application.ProcessMessages;
	Globe.Globefile := ExtractFilePath( Paramstr(0)) + 'Globe2.Dat';

	Globe.Brush.Color := clPurple;
	Globe.Font.Color := clBlack;
	MyObject := TGlobeTriangle.Create( Globe.overlays[5], 'Globe Triangle', 0, 0, 0 );

	UpdateLocationList( Sender );
end;

{-------------------------------------------------------------------------
	TFrmDemo.IdleHandler
-------------------------------------------------------------------------}
procedure TFrmDemo.IdleHandler(Sender: TObject; var Done: Boolean);
begin
	with Globe.Overlays[0] do
	begin
		LayerEnabled[1] := cbxLayer1.Checked;
		LayerEnabled[3] := cbxLayer3.Checked;
		LayerEnabled[4] := cbxLayer4.Checked;
	end;
	ShowMousePosition;

	if MyObject <> nil then
		MyObject.Origin := Point( iLong, Trunc( 60 * ONE_DEGREE * Cos( AngleToRadians( iLong ))));

	iLong := iLong + 5;
	if iLong > DEGREES_180 then
		iLong := -DEGREES_180;
	Globe.Invalidate;

	Done := True;
end;

{-------------------------------------------------------------------------
	TFrmDemo.UpdateGlobe
-------------------------------------------------------------------------}
procedure TFrmDemo.UpdateGlobe;
begin
	with Globe do
	begin
		if not ShowGrid1.Checked then
				DrawGrid := dgNone
		else
			if Gridinfront1.Checked then
				DrawGrid := dgInFront
			else
				DrawGrid := dgBehind;

		if MercatorProjection1.Checked then
			Projection := gpMercator
		else
			Projection := gpSpherical;
	end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.GlobeMouseDown
-------------------------------------------------------------------------}
procedure TFrmDemo.GlobeMouseDown(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	with Globe do
		if Button = mbLeft then
		begin
			SelectedObject := ObjectAtXY( X, Y );	{ Set the Globe SelectedObject }
			bMouseDown := true;
			DeviceXYToLL( X, Y, ptLoc );
		end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.GlobeMouseMove
-------------------------------------------------------------------------}
procedure TFrmDemo.GlobeMouseMove(Sender: TObject; Shift: TShiftState;
	X, Y: Integer);
var
	pt : TPoint;
begin
	iMouseX := X;
	iMouseY := Y;
	pt := ShowMousePosition;

	if bMouseDown then
		with Globe do
		begin
			if ssCtrl in Shift then
				if SelectedObject <> nil then
				begin
					SelectedObject.Origin := pt;
					Exit;
				end;

			if pt.Y <= DEGREES_90 then
			begin
				YRotationInMins := YRotationInMins + ( pt.X - ptLoc.X );
				XRotationInMins := XRotationInMins - ( pt.Y - ptLoc.Y );
				Overlays[0].LayerEnabled[1] := false;
				Overlays[0].LayerEnabled[3] := false;
				Overlays[0].LayerEnabled[4] := false;
			end;
		end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.GlobeMouseUp
-------------------------------------------------------------------------}
procedure TFrmDemo.GlobeMouseUp(Sender: TObject; Button: TMouseButton;
	Shift: TShiftState; X, Y: Integer);
begin
	bMouseDown := false;
end;

{-------------------------------------------------------------------------
	TFrmDemo.cbxLayer0Click
-------------------------------------------------------------------------}
procedure TFrmDemo.cbxLayer0Click(Sender: TObject);
begin
	with TCheckBox( Sender ) do
		Globe.Overlays[0].LayerEnabled[Tag] := Checked;
end;

{-------------------------------------------------------------------------
	TFrmDemo.btnDeleteClick
-------------------------------------------------------------------------}
procedure TFrmDemo.btnDeleteClick(Sender: TObject);
begin
	with Globe do
		if SelectedObject <> nil then
			SelectedObject.Delete;
end;

{-------------------------------------------------------------------------
	TFrmDemo.btnUpClick
-------------------------------------------------------------------------}
procedure TFrmDemo.btnUpClick(Sender: TObject);
begin
	Globe.XRotation := Globe.XRotation - 5;
end;

{-------------------------------------------------------------------------
	TFrmDemo.btnDownClick
-------------------------------------------------------------------------}
procedure TFrmDemo.btnDownClick(Sender: TObject);
begin
	Globe.XRotation := Globe.XRotation + 5;
end;

{-------------------------------------------------------------------------
	TFrmDemo.btnRightClick
-------------------------------------------------------------------------}
procedure TFrmDemo.btnRightClick(Sender: TObject);
begin
	Globe.YRotation := Globe.YRotation + 5;
end;

{-------------------------------------------------------------------------
	TFrmDemo.btnLeftClick
-------------------------------------------------------------------------}
procedure TFrmDemo.btnLeftClick(Sender: TObject);
begin
	Globe.YRotation := Globe.YRotation - 5;
end;

{-------------------------------------------------------------------------
	TFrmDemo.FormKeyDown
-------------------------------------------------------------------------}
procedure TFrmDemo.FormKeyDown(Sender: TObject; var Key: Word;
	Shift: TShiftState);
begin
	if not ( ssCtrl in Shift ) then
		Exit;

	case Key of
	VK_UP :    btnUpClick( Sender );
	VK_RIGHT : btnRightClick( Sender );
	VK_DOWN :  btnDownClick( Sender );
	VK_LEFT :  btnLeftClick( Sender );
	else
		Exit;
	end;
	Key := 0;
end;

{-------------------------------------------------------------------------
	TFrmDemo.SpinLatLongChange
-------------------------------------------------------------------------}
procedure TFrmDemo.SpinLatLongChange(Sender: TObject);
begin
	Globe.LatitudeStep := SpinLatitude.Value;
	Globe.LongitudeStep := SpinLongitude.Value;
end;

{-------------------------------------------------------------------------
	TFrmDemo.btnFillClick
-------------------------------------------------------------------------}
procedure TFrmDemo.btnFillClick(Sender: TObject);
begin
	with Globe do
		if ColorDialog1.Execute then
		begin
			with TGlobePolygon( SelectedObject ) do
				ObjectBrush := TGlobeBrush.Define( SelectedObject.Overlay, ColorDialog1.Color, Brush.Style );
			Invalidate;
		end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.btnPenClick
-------------------------------------------------------------------------}
procedure TFrmDemo.btnPenClick(Sender: TObject);
begin
	with Globe do
		if ColorDialog1.Execute then
		begin
			with TGlobePolyLine( SelectedObject ) do
				ObjectPen := TGlobePen.Define( SelectedObject.Overlay, ColorDialog1.Color, Pen.Style );

			Invalidate;
		end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.btnExtentsClick
-------------------------------------------------------------------------}
procedure TFrmDemo.btnExtentsClick(Sender: TObject);
begin
	Globe.ViewRect := Rect( 0,0,0,0 );
end;

{-------------------------------------------------------------------------
	TFrmDemo.EditTitleChange
-------------------------------------------------------------------------}
procedure TFrmDemo.EditTitleChange(Sender: TObject);
begin
	if Globe.SelectedObject <> nil then
		Globe.SelectedObject.Title := editTitle.text;
end;

{-------------------------------------------------------------------------
	TFrmDemo.GlobeRender
-------------------------------------------------------------------------}
procedure TFrmDemo.GlobeRender(Sender: TObject);
var
	ptA, ptB : Tpoint;
	iRadius, iLatA, iLatB : integer;
begin
exit;
	with Globe do
	begin
		{ displays a scaled 50 minute radius moving circle }
		iRadius := ScaleMinutesToDevice( 50 ) + 1;

		iLatA := Trunc( 45 * ONE_DEGREE * Sin( AngleToRadians( iLong )));
		if LLtoDeviceXY( iLong, iLatA, ptA ) then	{ True if point visible }
			with ptA do
			begin
				GlobeCanvas.Brush.Color := clRed;
				GlobeCanvas.ellipse( X - iRadius, Y - iRadius, X + iRadius, Y + iRadius );
			end;

		{ display second point }
		iLatB := Trunc( 45 * ONE_DEGREE * Cos( AngleToRadians( iLong )));	{ convert minutes to Radians }
		if LLtoDeviceXY( -iLong, iLatB, ptB ) then	{ True if point visible }
			with ptB do
			begin
				GlobeCanvas.Brush.Color := clYellow;
				GlobeCanvas.ellipse( X - iRadius, Y - iRadius, X + iRadius, Y + iRadius );
			end;

		{ joins points }
		RenderLine( Point( iLong, iLatA ), Point( -iLong, iLatB ), 32 );
	end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.GlobeZoomed
-------------------------------------------------------------------------}
procedure TFrmDemo.GlobeZoomed(Sender: TObject);
begin
	{ enable / disable text layer dependent on zoom scale factor }
{	Globe.Overlays[0].LayerEnabled[4] := Globe.ScaleFactor > 0.5;}
end;

{-------------------------------------------------------------------------
	TFrmDemo.Image1Click
-------------------------------------------------------------------------}
procedure TFrmDemo.Image1Click(Sender: TObject);
begin
	NoteBook1.ActivePage := 'Globe';
end;

{-------------------------------------------------------------------------
	TFrmDemo.MercatorProjection1Click
-------------------------------------------------------------------------}
procedure TFrmDemo.MercatorProjection1Click(Sender: TObject);
begin
	MercatorProjection1.Checked := not MercatorProjection1.Checked;
	UpdateGlobe;
end;

{-------------------------------------------------------------------------
	TFrmDemo.ShowGrid1Click
-------------------------------------------------------------------------}
procedure TFrmDemo.ShowGrid1Click(Sender: TObject);
begin
	ShowGrid1.Checked := not ShowGrid1.Checked;
	UpdateGlobe;
end;

{-------------------------------------------------------------------------
	TFrmDemo.Gridinfront1Click
-------------------------------------------------------------------------}
procedure TFrmDemo.Gridinfront1Click(Sender: TObject);
begin
	Gridinfront1.Checked := not Gridinfront1.Checked;
	UpdateGlobe;
end;

{-------------------------------------------------------------------------
	TFrmDemo.Exit1Click
-------------------------------------------------------------------------}
procedure TFrmDemo.Exit1Click(Sender: TObject);
begin
	Close;
end;

{-------------------------------------------------------------------------
	TFrmDemo.LoadOverlayClick
-------------------------------------------------------------------------}
procedure TFrmDemo.LoadOverlayClick(Sender: TObject);
begin
	with TMenuItem( Sender ) do
	try
		if Length( Globe.Overlays[Tag].Filename ) > 0 then
			Globe.Overlays[Tag].Clear
		else
		begin
			OpenDialog1.FileName := Globe.Overlays[Tag].Filename;
			if OpenDialog1.Execute then
				Globe.Overlays[Tag].LoadFromFile( OpenDialog1.FileName );
		end;
	finally
		Checked := Globe.Overlays[Tag].ObjectCount > 0;
	end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.Help1Click
-------------------------------------------------------------------------}
procedure TFrmDemo.Help1Click(Sender: TObject);
begin
	NoteBook1.ActivePage := 'Help';
	Image2.Picture.Bitmap.Assign( Globe.Bitmap );
end;

{-------------------------------------------------------------------------
	TFrmDemo.Button1Click
-------------------------------------------------------------------------}
procedure TFrmDemo.Button1Click(Sender: TObject);
begin
	NoteBook1.ActivePage := 'Globe';
end;

{-------------------------------------------------------------------------
	TFrmDemo.Save1Click
-------------------------------------------------------------------------}
procedure TFrmDemo.Save1Click(Sender: TObject);
begin
	SaveDialog1.FileName := Globe.Overlays[0].Filename;
	if SaveDialog1.Execute then
		Globe.Overlays[0].SaveToFile( SaveDialog1.FileName );
end;

{-------------------------------------------------------------------------
	TFrmDemo.btnBitmapClick
-------------------------------------------------------------------------}
procedure TFrmDemo.btnBitmapClick(Sender: TObject);
begin
	Image1.Picture.Bitmap.Assign( Globe.Bitmap );
	NoteBook1.ActivePage := 'Image';
end;

{-------------------------------------------------------------------------
	TFrmDemo.GlobeSelected
-------------------------------------------------------------------------}
procedure TFrmDemo.GlobeSelected(Sender: TObject);
begin
	with Globe do
	begin
		btnDelete.Enabled := ( SelectedObject <> nil );
		btnFill.Enabled := ( SelectedObject <> nil ) and ( SelectedObject is TGLobePolygon );
		btnPen.Enabled := ( SelectedObject <> nil );

		if SelectedObject <> nil then
			with SelectedObject do
			begin
				lstLocations.ItemIndex := lstLocations.Items.IndexOfObject( SelectedObject );
				PanelInfo.Caption := Format( ' %s  Origin %s  Overlay %d  Layer %d  Index %d  Title %s',
					[ ClassName, LLAsString( Origin.X, Origin.Y ), OverlayIndex, Layer, Index, Title] );
				EditTitle.Text := Title;
			end
			else
			begin
				lstLocations.ItemIndex := -1;
				PanelInfo.Caption := '';
				editTitle.text := '';
			end;
	end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.spnZoomUpClick
-------------------------------------------------------------------------}
procedure TFrmDemo.spnZoomUpClick(Sender: TObject);
begin
	with Globe do
		ScaleFactor := ScaleFactor * 1.1;
end;

{-------------------------------------------------------------------------
	TFrmDemo.spnZoomDownClick
-------------------------------------------------------------------------}
procedure TFrmDemo.spnZoomDownClick(Sender: TObject);
begin
	with Globe do
		ScaleFactor := ScaleFactor / 1.1;
end;

{-------------------------------------------------------------------------
	TFrmDemo.lstLocationsClick
-------------------------------------------------------------------------}
procedure TFrmDemo.lstLocationsClick(Sender: TObject);
begin
	with Globe do
	begin
		SelectedObject := TGlobeObject( lstLocations.Items.Objects[lstLocations.ItemIndex] );
		ObjectLocate( SelectedObject );
	end;
end;

{-------------------------------------------------------------------------
	TFrmDemo.UpdateLocationList
-------------------------------------------------------------------------}
procedure TFrmDemo.UpdateLocationList( Sender : TObject );
var
	idx, jdx : integer;
	bStore : Boolean;
begin
	lstLocations.Sorted := False;
	lstLocations.Clear;
	lstLocations.Items.BeginUpdate;
	with Globe do
		for idx := 0 to 10 do
			with Overlays[idx] do
				for jdx := 0 to ObjectCount - 1 do
				begin
					if Objects[jdx] is TGlobePolygon then
						bStore := cbxPolygon.checked
					else
						if Objects[jdx] is TGlobePolyline then
							bStore := cbxPolyline.checked
						else
							bStore := cbxPoint.checked;
					if bStore then
						lstLocations.Items.AddObject( Objects[jdx].Title, Objects[jdx] )
				end;
	lstLocations.Sorted := True;
	lstLocations.Items.EndUpdate;
end;

{-------------------------------------------------------------------------
	TFrmDemo.ShowHints1Click
-------------------------------------------------------------------------}
procedure TFrmDemo.ShowHints1Click(Sender: TObject);
begin
	frmDemo.ShowHint := not frmDemo.ShowHint;
	ShowHints1.Checked := frmDemo.ShowHint;
end;


end.
